==> subtitlecomposer-0.4.1.tar.bz2: subtitlecomposer-0.4.1/src/core/subtitleiterator.cpp <==
/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "subtitleiterator.h"
#include "subtitleline.h"

using namespace SubtitleComposer;

SubtitleIterator::SubtitleIterator( const Subtitle& subtitle, const RangeList& ranges, bool gotoLast ):
	m_ranges( ranges ),
	m_linesIterator( subtitle.m_lines ),
	m_linesIteratorStart( subtitle.m_lines )
{
	if ( subtitle.m_lines.count() )
		m_ranges.trimToIndex( subtitle.m_lines.count() - 1 );

	if ( m_ranges.empty() )
	{
		m_index = INVALID; // no operations allowed
		if ( subtitle.m_lines.count() )
			kdDebug() << "SubtitleIterator requested on empty ranges list" << endl;
	}
	else
	{
		if ( gotoLast )
			toLast();
		else
			toFirst();
	}
}

SubtitleIterator::SubtitleIterator( const SubtitleIterator& it ):
	m_ranges( it.m_ranges ),
	m_index( it.m_index ),
	m_rangesIterator( it.m_rangesIterator ),
	m_linesIterator( it.m_linesIterator ),
	m_linesIteratorStart( it.m_linesIteratorStart )
{
}

SubtitleIterator& SubtitleIterator::operator=( const SubtitleIterator& it )
{
	if ( &it != this )
	{
		m_ranges = it.m_ranges;
		m_index = it.m_index;
		m_rangesIterator = it.m_rangesIterator;
		m_linesIterator = it.m_linesIterator;
		m_linesIteratorStart = it.m_linesIteratorStart;
	}

	return *this;
}

SubtitleIterator::~SubtitleIterator()
{
}

int SubtitleIterator::firstIndex()
{
	return m_index == INVALID ? -1 : (signed)m_ranges.firstIndex();
}

int SubtitleIterator::lastIndex()
{
	return m_index == INVALID ? -1 : (signed)m_ranges.lastIndex();
}

void SubtitleIterator::toFirst()
{
	if ( m_index == INVALID )
		return;

	m_rangesIterator = m_ranges.begin();

	m_index = m_ranges.firstIndex();
	m_linesIterator = m_linesIteratorStart;
	if ( m_index )
		m_linesIterator += m_index; // safe because indexes within m_ranges are all valid lines
}

void SubtitleIterator::toLast()
{
	if ( m_index == INVALID )
		return;

	m_rangesIterator = m_ranges.end();
	m_rangesIterator--; // safe because m_ranges is not empty (otherwise m_index would be INVALID).

	m_index = m_ranges.lastIndex();
	m_linesIterator = m_linesIteratorStart;
	if ( m_index )
		m_linesIterator += m_index; // safe because indexes within m_ranges are all valid lines
}

bool SubtitleIterator::toIndex( unsigned index )
{
	if ( m_index == INVALID )
		return false;

	int signedIndex = (signed)index;

	if ( m_index == signedIndex )
		return true;

	if ( ! m_ranges.contains( index ) )
		return false;

	if ( m_index < signedIndex )
	{
		while ( m_index != signedIndex )
			operator++();
	}
	else
	{
		while ( m_index != signedIndex )
			operator--();
	}

	return true;
}

SubtitleIterator& SubtitleIterator::operator++()
{
	if ( m_index == INVALID || m_index == AFTER_LAST )
		return *this;

	if ( m_index == BEHIND_FIRST )
		toFirst();
	else
	{
		m_index++;
		++m_linesIterator;

		int currentRangeEnd = (*m_rangesIterator).end();
		if ( m_index > currentRangeEnd )
		{
			m_rangesIterator++;
			if ( m_rangesIterator == m_ranges.end() )
				m_index = AFTER_LAST;
			else
			{
				m_index = (*m_rangesIterator).start();
				m_linesIterator += (m_index - (currentRangeEnd + 1));
			}
		}
	}

	return *this;
}


SubtitleIterator& SubtitleIterator::operator--()
{
	if (  m_index == INVALID || m_index == BEHIND_FIRST )
		return *this;

	if ( m_index == AFTER_LAST )
		toFirst();
	else
	{
		m_index--;
		--m_linesIterator;

		int currentRangeStart = (*m_rangesIterator).start();
		if ( m_index < currentRangeStart )
		{
			if ( m_rangesIterator == m_ranges.begin() )
				m_index = BEHIND_FIRST;
			else
			{
				m_rangesIterator--;
				m_index = (*m_rangesIterator).end();
				m_linesIterator -= (currentRangeStart - (m_index + 1));
			}
		}
	}

	return *this;
}

SubtitleIterator& SubtitleIterator::operator+=( int count )
{
	if ( count > 0 )
	{
		for ( int i = 0; i < count; ++i )
			operator ++();
	}
	else if ( count < 0 )
	{
		for ( int i = 0; i > count; --i )
			operator --();
	}

	return *this;
}

SubtitleIterator& SubtitleIterator::operator-=( int count )
{
	if ( count > 0 )
	{
		for ( int i = 0; i < count; ++i )
			operator --();
	}
	else if ( count < 0 )
	{
		for ( int i = 0; i > count; --i )
			operator ++();
	}

	return *this;
}


bool SubtitleIterator::operator==( const SubtitleIterator& it )
{
	return m_index == it.m_index;
}

==> subtitlecomposer-0.4.2.tar.bz2: subtitlecomposer-0.4.2/src/core/subtitleiterator.cpp <==
/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "subtitleiterator.h"
#include "subtitleline.h"

using namespace SubtitleComposer;

SubtitleIterator::SubtitleIterator( const Subtitle& subtitle, const RangeList& ranges, bool gotoLast ):
	m_ranges( ranges ),
	m_linesIterator( subtitle.m_lines ),
	m_linesIteratorStart( subtitle.m_lines )
{
	if ( subtitle.m_lines.count() )
		m_ranges.trimToIndex( subtitle.m_lines.count() - 1 );
	else
		m_ranges.clear();

	if ( m_ranges.empty() )
	{
		m_index = INVALID; // no operations allowed
		if ( subtitle.m_lines.count() )
			kdDebug() << "SubtitleIterator requested on empty ranges list or empty subtitle" << endl;
	}
	else
	{
		if ( gotoLast )
			toLast();
		else
			toFirst();
	}
}

SubtitleIterator::SubtitleIterator( const SubtitleIterator& it ):
	m_ranges( it.m_ranges ),
	m_index( it.m_index ),
	m_rangesIterator( it.m_rangesIterator ),
	m_linesIterator( it.m_linesIterator ),
	m_linesIteratorStart( it.m_linesIteratorStart )
{
}

SubtitleIterator& SubtitleIterator::operator=( const SubtitleIterator& it )
{
	if ( &it != this )
	{
		m_ranges = it.m_ranges;
		m_index = it.m_index;
		m_rangesIterator = it.m_rangesIterator;
		m_linesIterator = it.m_linesIterator;
		m_linesIteratorStart = it.m_linesIteratorStart;
	}

	return *this;
}

SubtitleIterator::~SubtitleIterator()
{
}

int SubtitleIterator::firstIndex()
{
	return m_index == INVALID ? -1 : (signed)m_ranges.firstIndex();
}

int SubtitleIterator::lastIndex()
{
	return m_index == INVALID ? -1 : (signed)m_ranges.lastIndex();
}

void SubtitleIterator::toFirst()
{
	if ( m_index == INVALID )
		return;

	m_rangesIterator = m_ranges.begin();

	m_index = m_ranges.firstIndex();
	m_linesIterator = m_linesIteratorStart;
	if ( m_index )
		m_linesIterator += m_index; // safe because indexes within m_ranges are all valid lines
}

void SubtitleIterator::toLast()
{
	if ( m_index == INVALID )
		return;

	m_rangesIterator = m_ranges.end();
	m_rangesIterator--; // safe because m_ranges is not empty (otherwise m_index would be INVALID).

	m_index = m_ranges.lastIndex();
	m_linesIterator = m_linesIteratorStart;
	if ( m_index )
		m_linesIterator += m_index; // safe because indexes within m_ranges are all valid lines
}

bool SubtitleIterator::toIndex( unsigned index )
{
	if ( m_index == INVALID )
		return false;

	int signedIndex = (signed)index;

	if ( m_index == signedIndex )
		return true;

	if ( ! m_ranges.contains( index ) )
		return false;

	if ( m_index < signedIndex )
	{
		while ( m_index != signedIndex )
			operator++();
	}
	else
	{
		while ( m_index != signedIndex )
			operator--();
	}

	return true;
}

SubtitleIterator& SubtitleIterator::operator++()
{
	if ( m_index == INVALID || m_index == AFTER_LAST )
		return *this;

	if ( m_index == BEHIND_FIRST )
		toFirst();
	else
	{
		m_index++;
		++m_linesIterator;

		int currentRangeEnd = (*m_rangesIterator).end();
		if ( m_index > currentRangeEnd )
		{
			m_rangesIterator++;
			if ( m_rangesIterator == m_ranges.end() )
				m_index = AFTER_LAST;
			else
			{
				m_index = (*m_rangesIterator).start();
				m_linesIterator += (m_index - (currentRangeEnd + 1));
			}
		}
	}

	return *this;
}


SubtitleIterator& SubtitleIterator::operator--()
{
	if (  m_index == INVALID || m_index == BEHIND_FIRST )
		return *this;

	if ( m_index == AFTER_LAST )
		toFirst();
	else
	{
		m_index--;
		--m_linesIterator;

		int currentRangeStart = (*m_rangesIterator).start();
		if ( m_index < currentRangeStart )
		{
			if ( m_rangesIterator == m_ranges.begin() )
				m_index = BEHIND_FIRST;
			else
			{
				m_rangesIterator--;
				m_index = (*m_rangesIterator).end();
				m_linesIterator -= (currentRangeStart - (m_index + 1));
			}
		}
	}

	return *this;
}

SubtitleIterator& SubtitleIterator::operator+=( int count )
{
	if ( count > 0 )
	{
		for ( int i = 0; i < count; ++i )
			operator ++();
	}
	else if ( count < 0 )
	{
		for ( int i = 0; i > count; --i )
			operator --();
	}

	return *this;
}

SubtitleIterator& SubtitleIterator::operator-=( int count )
{
	if ( count > 0 )
	{
		for ( int i = 0; i < count; ++i )
			operator --();
	}
	else if ( count < 0 )
	{
		for ( int i = 0; i > count; --i )
			operator ++();
	}

	return *this;
}


bool SubtitleIterator::operator==( const SubtitleIterator& it )
{
	return m_index == it.m_index;
}
